#### Program to read TRICKS output files 
#### (Originally created to read MAGICKAL)
#### Looks for molecule and plots abundance w.r.t. time
#### Created: 3/28/24
#### Author: Samantha Scibelli
#### Modified: 12/6/24
#### Modified by: Samantha Scibelli, R T Garrod

#######################################
######### USER INPUTS TO EDIT #########
#######################################

plot_title = 'Plot: Add title'

pathname = './'
readfile = 'top_output.d' 

### Upper and lower time values in plot
xlower=1e+2
xupper=1e+8

### Upper and lower abundance values in plot
ylower=1e-15
yupper=1e0

### Linestyles
line1 = '-' 
line2 = '-' 
line3 = '-'
line4 = '-'
line5 = '-'


#######################################
######## Define molecules #############
#######################################

## THIS VERSION ASKS USER FOR molecule 1, 2, and 3 to plot
print(' ')
mol1 =  input("Molecule 1 to plot: ")
mol1sp=' '+mol1+' '
color1 = 'black' #input("color molecule 1: ")
mol2 =  input("Molecule 2 to plot: ")
mol2sp=' '+mol2+' '
color2 = 'firebrick' #input("color molecule 2: ")
mol3 =  input("Molecule 3 to plot: ")
mol3sp=' '+mol3+' '
color3 = 'skyblue' #input("color molecule 3: ")
mol4 =  input("Molecule 4 to plot: ")
mol4sp=' '+mol4+' '
color4 = 'magenta' #input("color molecule 4: ")
mol5 =  input("Molecule 5 to plot: ")
mol5sp=' '+mol5+' '
color5 = 'goldenrod' #input("color molecule 5: ")
print(' ')


#######################################
######## Import packages ##############
#######################################

import numpy as np
from matplotlib import pyplot as plt

#######################################
######## MAIN PROGRAM    ##############
#######################################

#### Open and read file 
file = open(pathname+readfile)
lines = file.readlines()
file.close()

### NOTE: Assumes all output files start with same line structure!! 
starting_line = 54 #106
if (lines[starting_line].strip().split()[-4] =='OUTPUTS,'):
    print('line formatting is good...')
else:
    print('*WARNING* this file is structured differently')
    
#initilize variables 

idust = []

# First check if freeze-out is switched on 
if lines[17].strip().split()[-3] == 'YES':
    IDUST = 1
    idust.append(IDUST)
    print('[freeze-out of gas to dust grains]')
else: 
    print('[no freeze-out]')
    

#Now, set additional variables based values in the file

#Number of chemical species
nsmax = int(lines[20].split()[-1].replace(')',""))
#Number of gas phase species
nsgas = int(lines[21].split()[-1].replace(')',""))
#Number of grain species
nsgrain = int(lines[22].split()[-1].replace(')',""))
#number of elements    
nemax = int(lines[23].split()[-1].replace(')',""))

# gas-to-dust ratio (by number density)
gd = float(lines[35].split()[-1].replace(')',""))#double

#Number of sites per grain 
tns = float(lines[40].split()[-1].replace(')',""))#double

#Number of outputs
nt =  int(lines[starting_line].split()[-1].replace(')',""))#double
tmax = nt

#Final output time
tfinal = float(lines[52].split()[-2].replace(')',""))#double

if (xupper > tfinal):
    xupper = tfinal


### Function to find molecule
def find_molecule(mol):
    lookup = ' '+mol+' '
    withlook = 't(yr)'
    MOL_POS = []
    NUM_POS = []
    ROW_MOL = -1
    MOL_FLAG=0
    DUMMY=0
    # FIND ALL LINES WITH MOLECULE NAME:
    with open(pathname+readfile) as myFile:
        for num, line in enumerate(myFile, 1):
            if lookup in line and withlook in line:
                #if withlook 
                #print('found at line:', num)
                NUM_POS.append(num)

    #ONLY LINE DEALING WITH ABUNDANCE
    for i in range(0,len(NUM_POS)):
            if lines[NUM_POS[i]-3].split()[-1] == 'ABUNDANCE':
#                print('here' , lines[NUM_POS[i]-3].split()[-1] )
#                print(NUM_POS[i])
                MOL_POS.append(NUM_POS[i])
                MOL_FLAG=1

    # FIND THE ROW THE MOL IS AT
    if (MOL_FLAG == 1):
        ROW_MOL = lines[MOL_POS[0]-1].split().index(mol.strip()) - 2 #remove two for IT and t(yr)
        MOL_FLAG=MOL_POS[0]
        DUMMY=lines[MOL_POS[0]-1].split()

    return MOL_FLAG, ROW_MOL, DUMMY


### molecule reader
def read_mol_sets(line_start, tmax):
    tyr = []
    set1 = np.zeros((tmax,10))
    for i in range(0,tmax):
        tyr.append(float(lines[line_start+i].split()[1]))
        set1[i,0] = (lines[line_start+i].split()[2])
        set1[i,1] = (lines[line_start+i].split()[3])
        set1[i,2] = (lines[line_start+i].split()[4])
        set1[i,3] = (lines[line_start+i].split()[5])
        set1[i,4] = (lines[line_start+i].split()[6])
        set1[i,5] = (lines[line_start+i].split()[7])
        set1[i,6] = (lines[line_start+i].split()[8])
        set1[i,7] = (lines[line_start+i].split()[9])
        set1[i,8] = (lines[line_start+i].split()[10])
        set1[i,9] = (lines[line_start+i].split()[11])
    
    return tyr, set1


### function to plot abund vs time
def plot_five_molecule(molecule1, label1, color1, line1, \
                       molecule2, label2, color2, line2, \
                       molecule3, label3, color3, line3, \
                       molecule4, label4, color4, line4, \
                       molecule5, label5, color5, line5,):

    fig, ax = plt.subplots(1, 1, figsize=[12,8])

    iplot1=1
    iplot2=1
    iplot3=1
    iplot4=1
    iplot5=1

    ax.tick_params(labelsize=14)
    ax.minorticks_on()
    ax.tick_params()
    ax.tick_params(bottom=True, top=True, left=True, right=True, axis='both',direction="in",which='minor',length=6,width=1)#,labelsize=18)
    ax.tick_params(bottom=True, top=True, left=True, right=True, axis='both',direction="in",which='major',length=8,width=1)#,labelsize=18)
    
    if (molecule1 != '' and find_molecule(molecule1)[1] != -1):
        PLOT1 = read_mol_sets(find_molecule(molecule1)[0], tmax)
        PLOT1_V_X = PLOT1[0]
        PLOT1_V_Y = PLOT1[1][:, find_molecule(molecule1)[1]]
        ax.plot(PLOT1_V_X, PLOT1_V_Y, c=color1, label=label1+': gas', linestyle=line1)
        print('Species',molecule1,'is good - plotting')

        if (find_molecule('J'+molecule1)[1] != -1):
            PLOT1_surf = read_mol_sets(find_molecule('J'+molecule1)[0], tmax)
            PLOT1_surf_V_X = PLOT1_surf[0]
            PLOT1_surf_V_Y = PLOT1_surf[1][:, find_molecule('J'+molecule1)[1]]
            ax.plot(PLOT1_surf_V_X, PLOT1_surf_V_Y, c=color1, label=label1+': surface', linestyle=':')
        else: print('Species',molecule1,'has no surface equivalent')
    else:
        print('Species',molecule1,'NOT FOUND - ignoring')
        iplot1=0 


    if (molecule2 != '' and find_molecule(molecule2)[1] != -1):
        PLOT2 = read_mol_sets(find_molecule(molecule2)[0], tmax)
        PLOT2_V_X = PLOT2[0]
        PLOT2_V_Y = PLOT2[1][:, find_molecule(molecule2)[1]]
        ax.plot(PLOT2_V_X, PLOT2_V_Y, c=color2, label=label2+': gas', linestyle=line2)
        print('Species',molecule2,'is good - plotting')

        if (find_molecule('J'+molecule2)[1] != -1):
            PLOT2_surf = read_mol_sets(find_molecule('J'+molecule2)[0], tmax)
            PLOT2_surf_V_X = PLOT2_surf[0]
            PLOT2_surf_V_Y = PLOT2_surf[1][:, find_molecule('J'+molecule2)[1]]
            ax.plot(PLOT2_surf_V_X, PLOT2_surf_V_Y, c=color2, label=label2+': surface', linestyle=':')
        else: print('Species',molecule2,'has no surface equivalent')
    else:
        print('Species',molecule2,'NOT FOUND - ignoring') 
        iplot2=0 


    if (molecule3 != '' and find_molecule(molecule3)[1] != -1):
        PLOT3 = read_mol_sets(find_molecule(molecule3)[0], tmax)
        PLOT3_V_X = PLOT3[0]
        PLOT3_V_Y = PLOT3[1][:, find_molecule(molecule3)[1]]
        ax.plot(PLOT3_V_X, PLOT3_V_Y, c=color3, label=label3+': gas', linestyle=line3)
        print('Species',molecule3,'is good - plotting')

        if (find_molecule('J'+molecule3)[1] != -1):
            PLOT3_surf = read_mol_sets(find_molecule('J'+molecule3)[0], tmax)
            PLOT3_surf_V_X = PLOT3_surf[0]
            PLOT3_surf_V_Y = PLOT3_surf[1][:, find_molecule('J'+molecule3)[1]]
            ax.plot(PLOT3_surf_V_X, PLOT3_surf_V_Y, c=color3, label=label3+': surface', linestyle=':')
        else: print('Species',molecule3,'has no surface equivalent')
    else:
        print('Species',molecule3,'NOT FOUND - ignoring') 
        iplot3=0 


    if (molecule4 != '' and find_molecule(molecule4)[1] != -1):
        PLOT4 = read_mol_sets(find_molecule(molecule4)[0], tmax)
        PLOT4_V_X = PLOT4[0]
        PLOT4_V_Y = PLOT4[1][:, find_molecule(molecule4)[1]]
        ax.plot(PLOT4_V_X, PLOT4_V_Y, c=color4, label=label4+': gas', linestyle=line4)
        print('Species',molecule4,'is good - plotting')

        if (find_molecule('J'+molecule4)[1] != -1):
            PLOT4_surf = read_mol_sets(find_molecule('J'+molecule4)[0], tmax)
            PLOT4_surf_V_X = PLOT4_surf[0]
            PLOT4_surf_V_Y = PLOT4_surf[1][:, find_molecule('J'+molecule4)[1]]
            ax.plot(PLOT4_surf_V_X, PLOT4_surf_V_Y, c=color4, label=label4+': surface', linestyle=':')
        else: print('Species',molecule4,'has no surface equivalent')
    else:
        print('Species',molecule4,'NOT FOUND - ignoring') 
        iplot4=0 


    if (molecule5 != '' and find_molecule(molecule5)[1] != -1):
        PLOT5 = read_mol_sets(find_molecule(molecule5)[0], tmax)
        PLOT5_V_X = PLOT5[0]
        PLOT5_V_Y = PLOT5[1][:, find_molecule(molecule5)[1]]
        ax.plot(PLOT5_V_X, PLOT5_V_Y, c=color5, label=label5+': gas', linestyle=line5)
        print('Species',molecule5,'is good - plotting')

        if (find_molecule('J'+molecule5)[1] != -1):
            PLOT5_surf = read_mol_sets(find_molecule('J'+molecule5)[0], tmax)
            PLOT5_surf_V_X = PLOT5_surf[0]
            PLOT5_surf_V_Y = PLOT5_surf[1][:, find_molecule('J'+molecule5)[1]]
            ax.plot(PLOT5_surf_V_X, PLOT5_surf_V_Y, c=color5, label=label5+': surface', linestyle=':')
        else: print('Species',molecule5,'has no surface equivalent')
    else:
        print('Species',molecule5,'NOT FOUND - ignoring') 
        iplot5=0 


    filename='TRICKS_'+str(readfile)
    if (iplot1 != 0):
        filename=filename+'_'+str(molecule1)
    if (iplot2 != 0):
        filename=filename+'_'+str(molecule2)
    if (iplot3 != 0):
        filename=filename+'_'+str(molecule3)
    if (iplot4 != 0):
        filename=filename+'_'+str(molecule4)
    if (iplot5 != 0):
        filename=filename+'_'+str(molecule5)
    filename=filename+'.png'


    ax.set_xlabel('Time (yr)', size=16)
    ax.set_ylabel('Fractional Abundance [n(i)/n$_\mathrm{H}$]', size=16)
    ax.set_xlim(xlower, xupper)
    ax.set_ylim(ylower, yupper)
    ax.set_yscale('log')
    ax.set_xscale('log')
    ax.legend(fontsize=14, loc='best', ncol=3, framealpha=1)
    plt.title(str(plot_title), color='grey')
    plt.tight_layout()

    plt.savefig(filename, dpi=1200)
    plt.show()



plot_five_molecule(str(mol1), str(mol1), str(color1), str(line1), \
                   str(mol2), str(mol2), str(color2), str(line2), \
                   str(mol3), str(mol3), str(color3), str(line3), \
                   str(mol4), str(mol4), str(color4), str(line4), \
                   str(mol5), str(mol5), str(color5), str(line5))


